@@ -53,7 +53,6 @@ stderr = getLogger('stderr.{}'.format(modname))
 if len(stderr.handlers) == 0:
     stderr.propagate = False
     stderr.setLevel(DEBUG)
-    #errlh = StreamHandler(stream=InlineStream(sys.stderr))
     errlh = StreamHandler(stream=InlineStream("stderr"))
     errlh.setFormatter(log_formatter)
     stderr.addHandler(errlh)
@@ -63,7 +62,6 @@ stdout = getLogger('stdout.{}'.format(modname))
 if len(stdout.handlers) == 0:
     stdout.propagate = False
     stdout.setLevel(DEBUG)
-    #outlh = StreamHandler(stream=InlineStream(sys.stdout))
     outlh = StreamHandler(stream=InlineStream("stdout"))
     outlh.setFormatter(log_formatter)
     stdout.addHandler(outlh)
@@ -75,15 +73,27 @@ class LevelFilter(object):
         self.__level = NOTSET
 
     def filter(self, logRecord):
-        #pout.v(logRecord.levelname[0], self.levels)
         return logRecord.levelname[0].upper() not in self.levels
-        #print(logRecord.levelname[0].upper())
-        #print(self.levels)
-        #return logRecord.levelname[0].upper() in self.levels
 
 
 class QuietFilter(String):
     """see --quiet flag help for what this does"""
+    @classmethod
+    def reset(cls):
+        """This will go through and remove all the filters that this class added
+        to all the logging handlers
+
+        This is mainly for testing
+        """
+        loggers = dict(Logger.manager.loggerDict)
+        for logger_name, logger in loggers.items():
+            # https://docs.python.org/3/library/logging.html#handler-objects
+            for handler in getattr(logger, "handlers", []):
+                for f in list(handler.filters):
+                    if isinstance(f, LevelFilter):
+                        #print(f"Removing {f} from {logger_name}")
+                        handler.removeFilter(f)
+
     def __new__(cls, levels, **kwargs):
         levels = levels or ""
         loggers = dict(Logger.manager.loggerDict)
